/***************************************************************************
                          mdkhtmlbrowser.cpp  -  description
                             -------------------
    begin                : mar jan 21 02:49:21 EST 2003
    copyright            : (C) 2003 by Montel Laurent
    email                : lmontel@mandriva.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qfile.h>

#include "mdkhtmlbrowser.h"
#include "mdkhtmlbrowser.moc"
#include <qcheckbox.h>
#include <kpushbutton.h>
#include <khtml_part.h>
#include <khtmlview.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qlayout.h>
#include <klocale.h>
#include <kseparator.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kdebug.h>
#include <krun.h>
#include <kcmdlineargs.h>
#include <kmessagebox.h>
#include <kwin.h>
#include <stdlib.h>
#include <kstdguiitem.h>

extern "C" {
#ifdef HAVE_XINERAMA
# include <X11/XKBlib.h>
# include <X11/extensions/Xinerama.h>
#endif
};


MdkhtmlbrowserApplication::MdkhtmlbrowserApplication()
    : KUniqueApplication()
{
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

    QString htmlPage;
    if( args->count() == 1 )
    {
        htmlPage=QString::fromLocal8Bit(args->arg(0));
    }

    Mdkhtmlbrowser *mdkhtmlbrowser = new Mdkhtmlbrowser(htmlPage);
    setMainWidget(mdkhtmlbrowser);
    mdkhtmlbrowser->show();
}

Mdkhtmlbrowser::Mdkhtmlbrowser(const QString & htmlPage, QWidget *parent, const char *name)
    : KDialog(parent, name)
{
  KWin::setIcons( winId(), kapp->icon(), kapp->miniIcon() );
	KGlobal::locale()->insertCatalogue("mdkhtmlbrowser");
    QVBoxLayout *vbox = new QVBoxLayout(this, marginHint(), spacingHint());
    QHBox *mBox = new QHBox(this);
    viewPart = new KHTMLPart(mBox,"khtml");
	//viewPart->view()->setVScrollBarMode(QScrollView::AlwaysOff);
	viewPart->view()->setHScrollBarMode(QScrollView::AlwaysOff);
    //define it.
    viewPart->setJScriptEnabled(true);
    viewPart->setJavaEnabled(false);
    viewPart->setMetaRefreshEnabled(false);
    viewPart->setPluginsEnabled(false);
    QWidget::connect(viewPart, SIGNAL(setWindowCaption(const QString &)),
		     this, SLOT(setCaption(const QString &)));

    if( htmlPage.isEmpty())
      {
	QString urlToOpen = QString("/usr/share/mdk/mandrivagalaxy/index-%1.html");
	QStringList langList = QStringList::split(":",QString::fromLocal8Bit(getenv("LANGUAGE")));
    bool languageFound = false;
	for ( QStringList::Iterator it = langList.begin(); it != langList.end(); ++it )
	  {
	    QString tmpUrl = urlToOpen.arg(*it);
	    if( QFile::exists(tmpUrl))
	      {
		viewPart->openURL(KURL( tmpUrl ));
		languageFound = true;
		break;
	      }
	  }
		if( !languageFound )
		{
			viewPart->openURL(KURL( "/usr/share/mdk/mandrivagalaxy/index.html" ));
		}
      }
    else
        viewPart->openURL(KURL( htmlPage ));
    vbox->addWidget( mBox );
    QHBox *hbox = new QHBox(this);
    hbox->setSpacing(0);
    hbox->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    vbox->addWidget(hbox);

    QHBoxLayout *hbox2 = new QHBoxLayout(vbox, 4);

    m_showAtTheNewStartUp=new QCheckBox( i18n("Open this window on startup"), this);
    m_showAtTheNewStartUp->setChecked(true );

    hbox2->addWidget(m_showAtTheNewStartUp, 1);

    m_close= new KPushButton( KStdGuiItem::close(), this );
    hbox2->addWidget(m_close);

    connect(m_close, SIGNAL(clicked()), this, SLOT(slotClose()));
    connect(m_showAtTheNewStartUp, SIGNAL(toggled(bool)), this, SLOT(showOnStart(bool)));
    connect(viewPart->browserExtension(),
            SIGNAL(openURLRequest(const KURL &, const KParts::URLArgs &)),
            this,
            SLOT(slotUrlOpen(const KURL &, const KParts::URLArgs &)));

	resize(860, 668);
    //xinerama support
    int dw, dh, gw, gh, x, y;
#ifdef HAVE_XINERAMA
    int numHeads;
    XineramaScreenInfo *xineramaInfo;
    if (XineramaIsActive(qt_xdisplay()) &&
        ((xineramaInfo = XineramaQueryScreens(qt_xdisplay(), &numHeads)))) {
	dw = xineramaInfo->width;
	dh = xineramaInfo->height;
	XFree(xineramaInfo);
    } else
#endif
    {
	dw = QApplication::desktop()->width();
	dh = QApplication::desktop()->height();
    }

    gw = width();
    gh = height();
    x = dw/2;
    y = dh/2;

    x -= gw/2;
    y -= gh/2;

    if (x + gw > dw)
	x = dw - gw;
    if (y + gh > dh)
	y = dh - gh;
    move( x < 0 ? 0 : x, y < 0 ? 0 : y );

    loadConfig();
}

Mdkhtmlbrowser::~Mdkhtmlbrowser()
{
}

void Mdkhtmlbrowser::loadConfig()
{

    KConfig* config=kapp->config();
    if( config->hasGroup("Parameter" ) )
    {
        config->setGroup( "Parameter" );
        m_showAtTheNewStartUp->setChecked(config->readBoolEntry("DontShowWindowAtTheNextStartUp",true));
    }
}

void Mdkhtmlbrowser::saveConfig()
{
    KConfig* config=kapp->config();
    config->setGroup( "Parameter" );
    config->writeEntry("DontShowWindowAtTheNextStartUp", m_showAtTheNewStartUp->isChecked() );
}


void Mdkhtmlbrowser::showOnStart(bool)
{
    saveConfig();
}

void Mdkhtmlbrowser::slotClose()
{
    saveConfig();
    accept();
}

void Mdkhtmlbrowser::slotUrlOpen(const KURL &url, const KParts::URLArgs &/*args*/)
{
    if( url.isEmpty() )
        return;

    bool htmlFile = url.fileName().contains(".html");
    bool docMdkFile = (url.url().contains("/usr/share/doc/mandrake"));

    if( !htmlFile && (url.protocol().isEmpty() || (url.protocol() == "file")))
    {
        (void) new KRun( url );
    }
    else if( htmlFile || url.protocol() == "http" || url.protocol() == "https")
    {
        if( docMdkFile )
        {
            QFile file( url.path());
            if( file.exists())
                openBrowser(url);
            else
            {
                int ret = KMessageBox::questionYesNo(this,i18n("The documentation isn't installed on your system. Do you want to install the needed packages (root password needed) ?"));
                if( (ret == 3 ))
                {
                    //launch drakehelp
                    if( !KRun::runCommand( "drakhelp --doc Starter.html/index.html") )
                        kdDebug()<<" we can launch drakehelp\n";
                }
            }
        }
        else
            openBrowser(url);
    }
}

void Mdkhtmlbrowser::openBrowser(const KURL &url)
{
    QString browser = QString(::getenv("BROWSER"));
    if(browser.isEmpty())
      browser="konqueror";
    browser+=" "+url.url();
    //kdDebug()<<" browser :"<<browser<<endl;
    KRun::runCommand( browser );
}
